set flows [lindex $argv 0]
set cbrrate [lindex $argv 1]
set queue [lindex $argv 2]
set rate [lindex $argv 3]
set time1 [lindex $argv 4]
set time2 [lindex $argv 5]
set stoptime [lindex $argv 6]
set flowtype [lindex $argv 7]
set discount [lindex $argv 8]
set numsample [lindex $argv 9]
set conservative [lindex $argv 10]
set maxHeavyRounds [lindex $argv 11]

puts "flows:$flows queue:$queue rate:$rate time1:$time1 time2:$time2 stoptime:$stoptime flowtype:$flowtype cbrrate:$cbrrate"

if {$flowtype=="TCP"||$flowtype=="Binomial"} {
    set increase [lindex $argv 8]
    set decrease [lindex $argv 9]  
    puts "increase: $increase decrease:$decrease"
}

if {$flowtype=="Binomial"} {
    set k_parameter [lindex $argv 10]
    set l_parameter [lindex $argv 11] 
}

if {$flowtype=="RAP"} {
    set increase [lindex $argv 8]
    set decrease [lindex $argv 9]  
    puts "increase: $increase decrease:$decrease"
    Agent/RAP set alpha_ $increase
    Agent/RAP set beta_ $decrease
}

# output files
set thru [open tmpthru w] 
set wf [open out.w w]

# basic bottleneck topology
set ns [new Simulator]
set node_(r1) [$ns node]
set node_(r2) [$ns node]
$ns duplex-link $node_(r1) $node_(r2) $rate 20ms $queue
set slink [$ns link $node_(r1) $node_(r2)]
set fmon [$ns makeflowmon Fid]
$ns attach-fmon $slink $fmon 

# Uncomment the following line to randomize runs using a random seed
ns-random 0     

#queue parameters
Queue/RED set gentle_ true
Queue/RED set setbit_ true
Agent/TCP set ecn_ 1

set redq [[$ns link $node_(r1) $node_(r2)] queue] 
set len [string length $rate]
set rate_w_mb [string range $rate 0 [expr $len-3]]
set limit [expr 2.5*$rate_w_mb*50/8.0]
$ns queue-limit $node_(r1) $node_(r2) $limit
if {$queue=="RED"} {
    $redq set thresh_ [expr 0.25*$rate_w_mb*50/8.0]
    $redq set maxthresh_ [expr 1.25*$rate_w_mb*50/8.0]
}
if {$queue=="DropTail"} {
    set limit [expr $rate_w_mb*50/8.0]
    $redq set thresh_ $limit
    $redq set maxthresh_ $limit
}
$redq set linterm_ 10  
puts "queue:$queue limit:$limit thresh_:[$redq set thresh_] maxthresh_:[$redq set maxthresh_]"  

# initialize drop variables
set bdrops [expr 0]
set bdepartures [expr 0]    
set flag 0
set area 0

proc compute_i_droprate {} {
    global bdrops bdepartures thru ns fmon rate_w_mb rdroprate idroprate
    global area flag
    set idrop [expr [$fmon set bdrops_]-$bdrops]
    set idepartures [expr [$fmon set bdepartures_]-$bdepartures]
    if {$idepartures > 0.0} {
	set idroprate [expr $idrop./($idepartures.+$idrop.)]
    } else {
	set idroprate [expr 0]
    }
    if {$flag==1} {
	set area [expr $area+$idrop]
	puts "[expr [$ns now]] [expr $area]"
    }
    set bdrops [$fmon set bdrops_]
    set bdepartures [$fmon set bdepartures_]
    set goodput [expr $idepartures.*5.0*8.0/($rate_w_mb*1000000)]
    puts $thru "[$ns now] $idrop $goodput $idroprate [$fmon set size_]"
    $ns at [expr [$ns now]+0.5] "compute_i_droprate"
}      

proc dropratemetric {} {
    global ns thru rdroprate idroprate time2 flag flowtype numsample area
    if {$idroprate > [expr 1.5*$rdroprate]} {
	set flag 1
    }
    if {$flag == 1} {
	if {$idroprate < [expr 1.5*$rdroprate]} {
	    puts $thru "$flowtype $numsample dropratetime [expr [$ns now]-$time2] $area"
	    exit
	}
    }
    $ns at [expr [$ns now]+0.5] "dropratemetric"
}

proc setlossrate {} {
    global bdrops bdepartures thru ns fmon rate_w_mb startdrop startdepartures
    set startdrop [expr [$fmon set bdrops_]]
    set startdepartures [expr [$fmon set bdepartures_]]
}

proc recordlossrate {} {
    global bdrops bdepartures thru ns fmon rate_w_mb startdrop startdepartures
    global rdroprate
    set rdrop [expr [$fmon set bdrops_]-$startdrop]
    set rdepartures [expr [$fmon set bdepartures_]-$startdepartures]
    if {$rdepartures > 0.0} {
	set rdroprate [expr $rdrop./($rdepartures.+$rdrop.)]
    } else {
	set rdroprate [expr 0]
    }
    puts $thru "rdroprate $rdroprate"
}

# Set up TCP connection
Agent/TFRC set packetSize_ 1000
Agent/TFRC set maxHeavyRounds_ $maxHeavyRounds
Agent/TFRC set scmult_ 1.1
Agent/TFRC set conservative_ $conservative
Agent/TFRC set df_ 0.95
Agent/TFRC set ca_ 1
Agent/TFRCSink set discount_ $discount
Agent/TFRCSink set smooth_ 1
Agent/TFRCSink set NumSamples_ $numsample

Agent/TCP/Sack1 set packetSize_ 1000
Agent/TCP/Sack1 set window_ 1000

puts "maxHeavyRounds: [Agent/TFRC set maxHeavyRounds_]"

for {set i 0} {$i < $flows} {incr i} {
    set node_(s$i) [$ns node]
    set node_(k$i) [$ns node]
    $ns duplex-link $node_(s$i) $node_(r1) 100Mb 2ms DropTail
    $ns duplex-link $node_(k$i) $node_(r2) 100Mb [expr $i/$flows]ms DropTail
    set sec [expr $i/$flows]
    set frac [expr $i%$flows+0.5]
    set starttime $sec.$frac

    if {$flowtype=="TFRC"} {
	set tf($i) [$ns create-connection TFRC $node_(s$i) TFRCSink $node_(k$i) 0]
	$ns at $starttime "[set tf($i)] start"
	$ns at $stoptime "[set tf($i)] stop"
    }
    if {$flowtype=="TCP"||$flowtype=="Binomial"} {
	set tf($i) [$ns create-connection TCP/Sack1 $node_(s$i) TCPSink/Sack1 $node_(k$i) 1]
	set ftp($i) [[set tf($i)] attach-app FTP]
	$tf($i) set increase_num_ $increase
	$tf($i) set decrease_num_ $decrease
	$tf($i) set precisionReduce_ true
	if {$flowtype=="Binomial"} {
	    $tf($i) set k_parameter_ $k_parameter
	    $tf($i) set l_parameter_ $l_parameter
	    $tf($i) set windowOption_ 6
	}
	$ns at $starttime "[set ftp($i)] start"
	$ns at $stoptime "[set ftp($i)] stop"
    }
    if {$flowtype=="RAP"} {
	set tf($i) [$ns create-connection RAP $node_(s$i) RAP $node_(k$i) 1]
	$ns at $starttime "[set tf($i)] start"
	$ns at $stoptime "[set tf($i)] stop"
    }
}

#configure cbr source
Application/Traffic/CBR set rate_ $cbrrate
puts "cbr rate [Application/Traffic/CBR set rate_]"
set udp0 [new Agent/UDP]
set node_(scbr) [$ns node]
set node_(kcbr) [$ns node]
$ns duplex-link $node_(scbr) $node_(r1) 100Mb 2ms DropTail
$ns duplex-link $node_(kcbr) $node_(r2) 100Mb 2ms DropTail
$ns attach-agent $node_(scbr) $udp0
set cbr(0) [new Application/Traffic/CBR]
$cbr(0) attach-agent $udp0
set null0 [new Agent/Null]
$ns attach-agent $node_(kcbr) $null0
$ns connect $udp0 $null0         

proc stop {} {
    global ns thru 
    $ns flush-trace
    close $thru
    exit
}

if {$flowtype=="TCP"||$flowtype=="Binomial"} {
    $tf(4) attach $wf
    $tf(4) trace cwnd_
}

set randomflows 4
if {$randomflows > $flows} {
    set randomflows $flows
}
for {set i 0} {$i < $randomflows} {incr i} {
    set tcp$i [$ns create-connection TCP/Sack1 $node_(k$i) TCPSink/Sack1 $node_(s$i) 1]
    [set tcp$i] set window_ 8
    set ftp$i [[set tcp$i] attach-app FTP]
    $ns at $i "[set ftp$i] start"
    $ns at $stoptime "[set ftp$i] stop"
}

$ns at 0 "$cbr(0) start"
$ns at $time1 "$cbr(0) stop"
$ns at $time2 "$cbr(0) start"

$ns at [expr $time1-10.0] "setlossrate"
$ns at $time1 "recordlossrate"
$ns at $time2 "dropratemetric"
$ns at 0.0 "compute_i_droprate"

$ns at $stoptime "$cbr(0) stop"
$ns at $stoptime "stop"

$ns run














