set queue [lindex $argv 0]
set rate [lindex $argv 1]
set rate1 [lindex $argv 2]
set time1 [lindex $argv 3]
set stoptime [lindex $argv 4]
set flowtype [lindex $argv 5]
set increase [lindex $argv 6]
set decrease [lindex $argv 7]
set k_param [lindex $argv 8]
set l_param [lindex $argv 9]
set discount [lindex $argv 10]
set numsample [lindex $argv 11]
set conservative [lindex $argv 12]
set maxHeavyRounds [lindex $argv 13]

puts "queue:$queue rate:$rate rate1:$rate1 time1:$time1 stoptime:$stoptime flowtype:$flowtype increase:$increase decrease:$decrease k_param:$k_param l_param:$l_param discount:$discount numsample:$numsample conservative:$conservative maxHeavyRounds:$maxHeavyRounds"

if {$flowtype=="TCP"||$flowtype=="Binomial"} {
    puts "increase: $increase decrease:$decrease"
    Agent/TCP set increase_num_ $increase
    Agent/TCP set decrease_num_ $decrease
    Agent/TCP set precisionReduce_ true
}

if {$flowtype=="Binomial"} {
    Agent/TCP set k_parameter_ $k_parameter
    Agent/TCP set l_parameter_ $l_parameter
    Agent/TCP set windowOption_ 6
}

 
set wf [open out.w w]
set wf1 [open out1.w w]
set tmpfair [open tmpfair w]

set ns [new Simulator]
set node_(r1) [$ns node]
set node_(r2) [$ns node]
$ns duplex-link $node_(r1) $node_(r2) $rate 20ms $queue

Queue/RED set gentle_ true
Queue/RED set setbit_ true
Agent/TCP set ecn_ 1

# Uncomment the following line to randomize runs using a random seed
ns-random 0     

set redq [[$ns link $node_(r1) $node_(r2)] queue] 
set len [string length $rate]
set rate_w_mb [string range $rate 0 [expr $len-3]]
set limit [expr 2.5*$rate_w_mb*50/8.0]
$ns queue-limit $node_(r1) $node_(r2) $limit
if {$queue=="RED"} {
    $redq set thresh_ [expr 0.25*$rate_w_mb*50/8.0]
    $redq set maxthresh_ [expr 1.25*$rate_w_mb*50/8.0]
}
if {$queue=="DropTail"} {
    $redq set thresh_ $limit
    $redq set maxthresh_ $limit
}
$redq set linterm_ 10  
puts "queue:$queue limit:$limit thresh_:[$redq set thresh_] maxthresh_:[$redq set maxthresh_]"  


# Set up TCP connection
Agent/TFRC set packetSize_ 1000
Agent/TFRC set maxHeavyRounds_ $maxHeavyRounds
Agent/TFRCSink set discount_ $discount
Agent/TFRCSink set smooth_ 1
Agent/TFRC set df_ 0.95
Agent/TFRC set ca_ 1
Agent/TFRCSink set NumSamples_ $numsample
Agent/TFRC set conservative_ $conservative

Agent/TCP/Sack1 set packetSize_ 1000
Agent/TCP/Sack1 set window_ 1000

puts "maxHeavyRounds: [Agent/TFRC set maxHeavyRounds_]"


set node_(s0) [$ns node]
set node_(k0) [$ns node]
$ns duplex-link $node_(s0) $node_(r1) 100Mb 2ms $queue
$ns duplex-link $node_(k0) $node_(r2) 100Mb 2ms $queue
set starttime 0.0

if {$flowtype=="TFRC"} {
    set tf(0) [$ns create-connection TFRC $node_(s0) TFRCSink $node_(k0) 0]
    $ns at $starttime "[set tf(0)] start"
    $ns at $stoptime "[set tf(0)] stop"
}
if {$flowtype=="TCP"||$flowtype=="Binomial"} {
    set tf(0) [$ns create-connection TCP/Sack1 $node_(s0) TCPSink/Sack1 $node_(k0) 1]
    set ftp(0) [[set tf(0)] attach-app FTP]
    $ns at $starttime "[set ftp(0)] start"
    $ns at $stoptime "[set ftp(0)] stop"
}

set node_(s1) [$ns node]
set node_(k1) [$ns node]
$ns duplex-link $node_(s1) $node_(r1) 15Mb 2ms $queue
$ns duplex-link $node_(k1) $node_(r2) 100Mb 2ms $queue
set starttime 0.0

if {$flowtype=="TFRC"} {
    set tf(1) [$ns create-connection TFRC $node_(s1) TFRCSink $node_(k1) 0]
    $ns at $starttime "[set tf(1)] start"
    $ns at $stoptime "[set tf(1)] stop"
}
if {$flowtype=="TCP"||$flowtype=="Binomial"} {
    set tf(1) [$ns create-connection TCP/Sack1 $node_(s1) TCPSink/Sack1 $node_(k1) 1]
    set ftp(1) [[set tf(1)] attach-app FTP]
    $ns at $starttime "[set ftp(1)] start"
    $ns at $stoptime "[set ftp(1)] stop"
}

#configure cbr source
Application/Traffic/CBR set rate_ 14.9Mb
puts "cbr rate [Application/Traffic/CBR set rate_]"
set udp0 [new Agent/UDP]
$ns attach-agent $node_(s1) $udp0
set cbr(0) [new Application/Traffic/CBR]
$cbr(0) attach-agent $udp0
set null0 [new Agent/Null]
set node_(kcbr) [$ns node]
$ns duplex-link $node_(r1) $node_(kcbr) 100Mb 20ms $queue
$ns attach-agent $node_(kcbr) $null0
$ns connect $udp0 $null0         

set flag 0
proc deltafair {} {
    global ns flag tf decrease tmpfair time1 ackcount1 ackcount2
    if {$flag==0} {
	set win1 [$tf(0) set cwnd_]
	set win2 [$tf(1) set cwnd_]
	if {$win2 > [expr ($win1+$win2)*0.45]} {
	    set flag 1
	    puts $tmpfair "deltafair [expr 1.0-$decrease] [expr [$ns now]-$time1] [expr [$tf(0) set ack_]-$ackcount1+[$tf(1) set ack_]-$ackcount2]"
	    exit
	}
    }
    $ns at [expr [$ns now]+0.1] "deltafair"
}

$ns at 0 "$cbr(0) start"
$ns at $time1 "$cbr(0) stop"
$ns at $time1 "set ackcount1 [$tf(0) set ack_]"
$ns at $time1 "set ackcount2 [$tf(1) set ack_]"
$ns at $time1 "deltafair"

proc stop {} {
    global ns wf wf1 
    $ns flush-trace
    close $wf
    close $wf1
    exit
}

if {$flowtype=="TCP"||$flowtype=="Binomial"} {
    $tf(0) attach $wf
    $tf(0) trace cwnd_
    $tf(1) attach $wf1
    $tf(1) trace cwnd_
}

set randomflows 2

for {set i 0} {$i < $randomflows} {incr i} {
    set tcp$i [$ns create-connection TCP/Sack1 $node_(k$i) TCPSink/Sack1 $node_(s$i) 1]
    [set tcp$i] set window_ 8
    set ftp$i [[set tcp$i] attach-app FTP]
    $ns at $i "[set ftp$i] start"
    $ns at $stoptime "[set ftp$i] stop"
}


$ns at $stoptime "stop"

$ns run














