set flows [lindex $argv 0]
set shortnumber [lindex $argv 1]
set queue [lindex $argv 2]
set rate [lindex $argv 3]
set time1 [lindex $argv 4]
set time2 [lindex $argv 5]
set stoptime [lindex $argv 6]
set flowtype [lindex $argv 7]

puts "no. of long flows:$flows queue type:$queue bw of bn:$rate time1:$time1 time2:$time2 stoptime:$stoptime flowtype:$flowtype number of flows in flash crowd:$shortnumber"

if {$flowtype=="TCP"||$flowtype=="iiad"||$flowtype=="sqrt"} {
    set increase [lindex $argv 8]
    set decrease [lindex $argv 9]  
    puts "increase: $increase decrease:$decrease"
}

if {$flowtype=="TFRC"} {
    set numsamples [lindex $argv 8]
    set conservative [lindex $argv 9]  
    puts "numsamples: $numsamples conservative:$conservative"
    Agent/TFRCSink set NumSamples_ $numsamples
    Agent/TFRC set conservative_ $conservative
}

Queue/RED set gentle_ true
Queue/RED set setbit_ true

# Set up TCP connection
Agent/TFRC set packetSize_ 1000
Agent/TFRC set scmult_ 1.1
Agent/TFRC set df_ 0.95
Agent/TFRC set ca_ 1
Agent/TFRCSink set discount_ 0
Agent/TFRCSink set smooth_ 1

Agent/TCP set precisionReduce_ 1
Agent/TCP set ecn_ 1
Agent/TCP/Sack1 set packetSize_ 1000
Agent/TCP/Sack1 set window_ 1000

set thru [open tmpthru w] 
set outwin [open out.wf w]

set ns [new Simulator]
set node_(r1) [$ns node]
set node_(r2) [$ns node]
$ns duplex-link $node_(r1) $node_(r2) $rate 20ms $queue
set slink [$ns link $node_(r1) $node_(r2)]
set fmon [$ns makeflowmon Fid]
$ns attach-fmon $slink $fmon 

set tmp [open tmp w]
$slink trace-dynamics $ns $tmp
# Uncomment the following line to randomize runs using a random seed
ns-random 0     

set redq [[$ns link $node_(r1) $node_(r2)] queue] 
set len [string length $rate]
set rate_w_mb [string range $rate 0 [expr $len-3]]
set limit [expr 2.5*$rate_w_mb*50./8.0]
$ns queue-limit $node_(r1) $node_(r2) $limit
if {$queue=="RED"} {
    $redq set thresh_ [expr 0.25*$rate_w_mb*50./8.0]
    $redq set maxthresh_ [expr 1.25*$rate_w_mb*50./8.0]
}
if {$queue=="DropTail"} {
    $redq set thresh_ $limit
    $redq set maxthresh_ $limit
}
$redq set linterm_ 10  
puts "queue:$queue limit:$limit thresh_:[$redq set thresh_] maxthresh_:[$redq set maxthresh_]"  

for {set i 0} {$i < $flows} {incr i} {
    set node_(s$i) [$ns node]
    set node_(k$i) [$ns node]
    $ns duplex-link $node_(s$i) $node_(r1) 100Mb 2ms DropTail
    $ns duplex-link $node_(k$i) $node_(r2) 100Mb [expr $i./$flows.]ms DropTail
    set sec [expr $i/$flows]
    set frac [expr $i%$flows+0.5]
    set starttime $sec.$frac

    if {$flowtype=="TFRC"} {
	set tf($i) [$ns create-connection TFRC $node_(s$i) TFRCSink $node_(k$i) 0]
	$ns at $starttime "[set tf($i)] start"
	$ns at $stoptime "[set tf($i)] stop"
    }
    if {$flowtype=="TCP"} {
	set tf($i) [$ns create-connection TCP/Sack1 $node_(s$i) TCPSink/Sack1 $node_(k$i) 1]
        $tf($i) set increase_num_ $increase
        $tf($i) set decrease_num_ $decrease  
	set ftp($i) [[set tf($i)] attach-app FTP]
	$ns at $starttime "[set ftp($i)] start"
	$ns at $stoptime "[set ftp($i)] stop"
    }
    if {$flowtype=="iiad"} {
	set tf($i) [$ns create-connection TCP/Sack1 $node_(s$i) TCPSink/Sack1 $node_(k$i) 1]
        $tf($i) set increase_num_ $increase
        $tf($i) set decrease_num_ $decrease
	$tf($i) set windowOption_ 6
	$tf($i) set k_parameter_ 1
	$tf($i) set l_parameter_ 0
	set ftp($i) [[set tf($i)] attach-app FTP]
	$ns at $starttime "[set ftp($i)] start"
	$ns at $stoptime "[set ftp($i)] stop"
    }

    if {$flowtype=="sqrt"} {
	set tf($i) [$ns create-connection TCP/Sack1 $node_(s$i) TCPSink/Sack1 $node_(k$i) 1]
        $tf($i) set increase_num_ $increase
        $tf($i) set decrease_num_ $decrease
	$tf($i) set windowOption_ 6
	$tf($i) set k_parameter_ 0.5
	$tf($i) set l_parameter_ 0.5
	set ftp($i) [[set tf($i)] attach-app FTP]
	$ns at $starttime "[set ftp($i)] start"
	$ns at $stoptime "[set ftp($i)] stop"
    }

}

$tf(0) attach $outwin
$tf(0) trace cwnd_

#flash crowd

set node_(sflash) [$ns node]
set node_(kflash) [$ns node]
$ns duplex-link $node_(sflash) $node_(r1) 100Mb 2ms DropTail
$ns duplex-link $node_(kflash) $node_(r2) 100Mb 2ms DropTail

proc flashtraffic {num window packets start interval } {
    global ns node_ flashtcp
    puts "interval [expr $interval]"
    for {set i 0} {$i < $num} {incr i} {
	set flashtcp($i) [$ns create-connection TCP/Sack1 $node_(sflash) TCPSink/Sack1 $node_(kflash) 1]
	$flashtcp($i) set window_ $window
	set flashftp($i) [$flashtcp($i) attach-app FTP]
	set time [expr $start+$i*$interval]
#	puts "scheduled shortflow [expr $i] at time [expr $time]"
	$ns at $time "$flashftp($i) produce $packets"
    }
}


# print output
set ackslong [expr 0]
set acksshort [expr 0]

set bdrops [expr 0]
set bdepartures [expr 0]

proc output {} {
    global tf flashtcp ns ackslong acksshort thru flows shortnumber flowtype
    global fmon bdrops bdepartures rate_w_mb
    set tmpackslong [expr 0]
    for {set i 0} {$i < $flows} {incr i} {
	if {$flowtype=="TFRC"} {
	    set tmpackslong [expr $tmpackslong+[$tf($i) set ndatapack_]]
	} else {
	    set tmpackslong [expr $tmpackslong+[$tf($i) set ack_]]
	}
    }
    set tmpacksshort [expr 0]
    for {set i 0} {$i < $shortnumber} {incr i} {
	set tmpacksshort [expr $tmpacksshort+[$flashtcp($i) set ack_]]
    }    
    set thrulong [expr ($tmpackslong-$ackslong)]
    set thrushort [expr ($tmpacksshort-$acksshort)]
    set ackslong [expr $tmpackslong]
    set acksshort [expr $tmpacksshort]

    set idrop [expr [$fmon set bdrops_]-$bdrops]
    set idepartures [expr [$fmon set bdepartures_]-$bdepartures]
    if {$idepartures > 0.0} {
	set idroprate [expr $idrop./($idepartures.+$idrop.)]
    } else {
	set idroprate [expr 0]
    }
    set bdrops [$fmon set bdrops_]
    set bdepartures [$fmon set bdepartures_]
    set goodput [expr $idepartures.*5.0*8.0/($rate_w_mb*1000000)]

    puts $thru "[$ns now] $thrulong $thrushort $idroprate $idrop $idepartures"
    $ns at [expr [$ns now]+1.0] "output"
}      


proc stop {} {
    global ns thru 
    $ns flush-trace
    close $thru
    exit
}

set randomflows 4
if {$randomflows > $flows} {
    set randomflows $flows
}
for {set i 0} {$i < $randomflows} {incr i} {
    set tcp$i [$ns create-connection TCP/Sack1 $node_(k$i) TCPSink/Sack1 $node_(s$i) 1]
    [set tcp$i] set window_ 8
    set ftp$i [[set tcp$i] attach-app FTP]
    $ns at $i "[set ftp$i] start"
    $ns at $stoptime "[set ftp$i] stop"
}


$ns at 1.0 "output"
flashtraffic $shortnumber 8 10 $time1 [expr ($time2.-$time1.)/$shortnumber.] 
$ns at $stoptime "stop"
$ns run








